@@ -141,7 +141,7 @@ sub register {
     ) for (qw/ .nan .NaN .NAN /);
     $schema->add_resolver(
         tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[2]->[0] } ],
+        match => [ all => sub { $_[1]->{value} } ],
         implicit => 0,
     );
 
@@ -252,7 +252,6 @@ YAML::PP::Schema::YAML1_1 - YAML 1.1 Schema for YAML::PP
 =head1 SYNOPSIS
 
     use YAML::PP;
-    use YAML::PP::Schema::YAML1_1;
 
     my $yp = YAML::PP->new( schema => ['YAML1_1'] );
     my $yaml = <<'EOM';
